from collections import namedtuple

# Nossa maquina por enquanto tem apenas
# memoria, posição atual de leitura
# e status
Maquina = namedtuple("Maquina", "memoria pos status")

# Os Status podem ser:
# IDLE: aguardando execução
# WAIT: esperando por entrada de dados (não utilizado no momento)
# HALT: terminou execução
IDLE =  0
WAIT =  1
HALT = -1

# Cria uma máquina a partir de uma lista com um estado de memória
def criaMaquina(xs : [int]) -> Maquina:
    return Maquina(xs.copy(), 0, IDLE)
         
def somar(m : Maquina) -> Maquina:
    memoria, pos, status = m
    
    '''
    Exercício 01:
    - Capture os valores na memória nas posições pos+1, pos+2 e pos+3
      vamos chamar de x, y, z
    - Utilize a sintaxe memoria[idx] para pegar a posição idx da memória
    - Na posição z da memória, some os valores das posições x e y
    - Como consumimos 4 valores (opcode, x, y, z) adicione 4 na variável pos
    '''
    x = memoria[pos+1]
    y = memoria[pos+2]
    z = memoria[pos+3]
    
    memoria[z] = memoria[x] + memoria[y]
    pos        = pos + 4
    '''FIM DO EXERCÍCIO'''
    
    return Maquina(memoria, pos, status)
    
def multiplicar(m : Maquina) -> Maquina:
    memoria, pos, status = m
    
    '''
    Exercício 02:
    - Siga os mesmos passos do exercício 01 porém troque a operação
      de soma por multiplicação
    '''
    x = memoria[pos+1]
    y = memoria[pos+2]
    z = memoria[pos+3]
    
    memoria[z] = memoria[x] * memoria[y]
    pos        = pos + 4
    '''FIM DO EXERCÍCIO'''
    
    return Maquina(memoria, pos, status)
    
# Termina a execução da máquina alterando seu status para HALT
def terminar(m : Maquina) -> Maquina:
    memoria, pos, status = m
    
    return Maquina(memoria, pos, HALT)
                
# Executa a próxima operação da máquina
def step(m : Maquina) -> Maquina:
    
    # pega o código de operação
    op = m.memoria[m.pos]
    
    # executa a função correspondente
    if   op == 1:
        return somar(m)
    elif op == 2:
        return multiplicar(m)
    elif op == 99:
        return terminar(m)
    return m

# executa a máquina até que, ou ela termine, ou ela aguarde por nova entrada
def executa(m : Maquina) -> (Maquina, [int]):
    memoria, pos, status = m
    m = Maquina(memoria, pos, status)   
    
    while m.status != WAIT and m.status != HALT:
        m = step(m)
    return m

# lê um arquivo de entrada
def leiaArquivo(nome : str) -> [int]:
    with open(nome) as f:
        return list(map(int, f.readlines()[0].split(",")))    
    

